unit BT848;
interface
  USES Windows,SSETUP;

CONST
   DTSTATUS     =$00;
   IFORM        =$04;
   TDEC         =$08;
   E_CROP       =$0C;
   O_CROP       =$8C;
   E_VDELAY_LO  =$10;
   O_VDELAY_LO  =$90;
   E_VACTIVE_LO =$14;
   O_VACTIVE_LO =$94;
   E_DELAY_LO   =$18;
   O_DELAY_LO   =$98;
   E_HACTIVE_LO =$1C;
   O_HACTIVE_LO =$9C;
   E_HSCALE_HI  =$20;
   O_HSCALE_HI  =$A0;
   E_HSCALE_LO  =$24;
   O_HSCALE_LO  =$A4;
   BRIGHT       =$28;
   E_CONTROL    =$2C;
   O_CONTROL    =$AC;
   CONTRAST_LO  =$30;
   SAT_U_LO     =$34;
   SAT_V_LO     =$38;
   HUE          =$3C;
   E_SCLOOP     =$40;
   O_SCLOOP     =$C0;
   OFORM        =$48;
   E_VSCALE_HI  =$4C;
   O_VSCALE_HI  =$CC;
   E_VSCALE_LO  =$50;
   O_VSCALE_LO  =$D0;
   TEST         =$54;
   ADELAY       =$60;
   BDELAY       =$64;
   ADC          =$68;
   E_VTC        =$6C;
   O_VTC        =$EC;
   SRESET       =$7C;
   COLOR_FMT    =$D4;
   COLOR_CTL    =$D8;
   CAP_CTL      =$DC;
   VBI_PACK_SIZE=$E0;
   VBI_PACK_DEL =$E4;
   PLL_F_LO     =$F0;
   PLL_F_HI     =$F4;
   PLL_XCI      =$F8;
   FCAP         =$E8;
   INT_STAT     =$100;
   INT_MASK     =$104;
   RISC_COUNT   =$120;
   RISC_STRT_ADD=$114;
   GPIO_DMA_CTL =$10C;
   GPIO_OUT_EN  =$118;
   GPIO_REG_INP =$11C;
   GPIO_DATA    =$200;
   I2C          =$110;

   rgb32=$00;
   rgb24=$11;
   rgb16=$22;
   rgb15=$33;
   yuy2=$44;
   btyuv=$55;
   y8=$66;
   rgb8=$77;
   yuy2p=$88;
   btyuvp=$99;
   raw8=$ee;

   //RISC COMMANDS
   RESYNCEVENTSTART=$8C00800C;
   RESYNCPACKEDDATA=$8C008006;
   RESYNCPLANARDATA=$8C00800E;
   SYNCODDSTART=$8C000004;    //SET RISC STATUS TO X AFTER EVENT END
   SYNCEVENTSTART=$8C00000C;  //SET RISC STATUS TO Y AFTER ODD END
   JUMP=$7C000000;
   WRITE768e  =$1CC30300;   //  PACKED
   WRITE768o  =$1C3C0300;   //   Y8

   WRITE123Y  =$9C000300;   //   PLANAR
   WRITE123UVC=$01800180;   //    4:2:2

   SKIPY1=$ACE10300;
   SKIPY2=$ACD20300;
   SKIPY3=$ACC30300;
   SKIPY4=$ACB40300;
   SKIPUV=$01800180;

TYPE BT848STRUC = ARRAY[0..$2FF] OF BYTE;

type BTBUFFERSTRUC = RECORD
                     EVENYBUFFER  : ARRAY[1..287,1..768] OF BYTE;
                     EVENCBBUFFER  : ARRAY[1..287,1..384] OF BYTE;
                     EVENCRBUFFER  : ARRAY[1..287,1..384] OF BYTE;
                     ODDYBUFFER   : ARRAY[1..287,1..768] OF BYTE;
                     ODDCBBUFFER   : ARRAY[1..287,1..384] OF BYTE;
                     ODDCRBUFFER   : ARRAY[1..287,1..384] OF BYTE;
                     EVENYBUFFER2  : ARRAY[1..287,1..768] OF BYTE;
                     EVENCBBUFFER2  : ARRAY[1..287,1..384] OF BYTE;
                     EVENCRBUFFER2  : ARRAY[1..287,1..384] OF BYTE;
                     ODDYBUFFER2   : ARRAY[1..287,1..768] OF BYTE;
                     ODDCBBUFFER2   : ARRAY[1..287,1..384] OF BYTE;
                     ODDCRBUFFER2   : ARRAY[1..287,1..384] OF BYTE;
                     END;

RISCPRG = ARRAY[0..8191] OF DWORD;

VAR
   BT848P  : POINTER;      //BT848 PHYSICAL ADDRESS
   BT848L  : ^BT848STRUC;  //BT848 LINEAR ADDRESS
   BT848IRQ : BYTE;

   VBASEL : POINTER;   //VIDEO LINEAR ADDRESS

   BTbufferp : ^BTBUFFERSTRUC;     //PHYSICAL ADDRESS
   BTbufferl : ^BTBUFFERSTRUC;      //LINEAR ADRESS

   RISCP     : ^RISCPRG;    //RISC PROGRAM PHYSICAL ADDRESS
   RISCL     : ^RISCPRG;    //RISC PROGRAM LINEAR ADDRESS

   RISCPTR:LONGINT;        //RISC IP
   VP : POINTER;

   INCAPTURE : BOOLEAN;

   EBUFFERADDR,OBUFFERADDR : POINTER;

   I,X,Y,Z : LONGINT;
   YCOUNT : DWORD;

   PROCEDURE WRITEBT(PT : POINTER;VALUE : DWORD);
   FUNCTION READBT(PT : POINTER) : DWORD;
   FUNCTION READSTAT:DWORD;
   PROCEDURE PREPARERISCPRG;
   PROCEDURE INITBT848;
   PROCEDURE CAPSTART;
   PROCEDURE CAPSTOP;

implementation
PROCEDURE PREPARERISCPRG;
BEGIN
{     RISC PROGRAM:
      RESYNC EVENTSTART
      RESYNC PACKEDDATA
      WRITE 256 LINES TO EBUFFER256
      WRITE 32 LINES TO OBUFFER32
      SYNC ODDSTART
      RESYNC PACKEDDATA
      WRITE 256 LINES TO OBUFFER256
      WRITE 32 LINES TO EBUFFER32
      SYNC EVENTSTART
      JMP RISCPROGRAM
}
RISCPTR:=0;
RISCL[RISCPTR]:=RESYNCEVENTSTART;
INC(RISCPTR);
RISCL[RISCPTR]:=0;
INC(RISCPTR);
RISCL[RISCPTR]:=RESYNCPLANARDATA;
INC(RISCPTR);
RISCL[RISCPTR]:=0;
INC(RISCPTR);
//PROCESS EVENT 256 +32 LINES;
FOR I:=33 TO 287 DO
    BEGIN
    RISCL[RISCPTR]:=WRITE123Y;
    INC(RISCPTR);
    RISCL[RISCPTR]:=WRITE123UVC;
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENYBUFFER[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENCBBUFFER[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENCRBUFFER[I,1]);
    INC(RISCPTR);
    END;
FOR I:=1 TO 32 DO
    BEGIN
    RISCL[RISCPTR]:=WRITE123Y;
    INC(RISCPTR);
    RISCL[RISCPTR]:=WRITE123UVC;
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDYBUFFER[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDCBBUFFER[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDCRBUFFER[I,1]);
    INC(RISCPTR);
    END;
    RISCL[RISCPTR]:=SKIPY1;
    INC(RISCPTR);
    RISCL[RISCPTR]:=SKIPUV;
    INC(RISCPTR);

RISCL[RISCPTR]:=SYNCODDSTART;
INC(RISCPTR);
RISCL[RISCPTR]:=0;
INC(RISCPTR);
RISCL[RISCPTR]:=RESYNCPLANARDATA;
INC(RISCPTR);
RISCL[RISCPTR]:=0;
INC(RISCPTR);
//PROCESS ODD 256 +32 LINES;
FOR I:=33 TO 287 DO
    BEGIN
    RISCL[RISCPTR]:=WRITE123Y;
    INC(RISCPTR);
    RISCL[RISCPTR]:=WRITE123UVC;
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDYBUFFER[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDCBBUFFER[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDCRBUFFER[I,1]);
    INC(RISCPTR);
    END;
FOR I:=1 TO 32 DO
    BEGIN
    RISCL[RISCPTR]:=WRITE123Y;
    INC(RISCPTR);
    RISCL[RISCPTR]:=WRITE123UVC;
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENYBUFFER2[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENCBBUFFER2[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENCRBUFFER2[I,1]);
    INC(RISCPTR);
    END;
    RISCL[RISCPTR]:=SKIPY2;
    INC(RISCPTR);
    RISCL[RISCPTR]:=SKIPUV;
    INC(RISCPTR);

RISCL[RISCPTR]:=SYNCEVENTSTART;
INC(RISCPTR);
RISCL[RISCPTR]:=0;
INC(RISCPTR);
RISCL[RISCPTR]:=RESYNCPLANARDATA;
INC(RISCPTR);
RISCL[RISCPTR]:=0;
INC(RISCPTR);
//PROCESS EVENT 256 +32 LINES;
FOR I:=33 TO 287 DO
    BEGIN
    RISCL[RISCPTR]:=WRITE123Y;
    INC(RISCPTR);
    RISCL[RISCPTR]:=WRITE123UVC;
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENYBUFFER2[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENCBBUFFER2[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENCRBUFFER2[I,1]);
    INC(RISCPTR);
    END;
FOR I:=1 TO 32 DO
    BEGIN
    RISCL[RISCPTR]:=WRITE123Y;
    INC(RISCPTR);
    RISCL[RISCPTR]:=WRITE123UVC;
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDYBUFFER2[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDCBBUFFER2[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDCRBUFFER2[I,1]);
    INC(RISCPTR);
    END;
    RISCL[RISCPTR]:=SKIPY3;
    INC(RISCPTR);
    RISCL[RISCPTR]:=SKIPUV;
    INC(RISCPTR);

RISCL[RISCPTR]:=SYNCODDSTART;
INC(RISCPTR);
RISCL[RISCPTR]:=0;
INC(RISCPTR);
RISCL[RISCPTR]:=RESYNCPLANARDATA;
INC(RISCPTR);
RISCL[RISCPTR]:=0;
INC(RISCPTR);
//PROCESS ODD 256 +32 LINES;
FOR I:=33 TO 287 DO
    BEGIN
    RISCL[RISCPTR]:=WRITE123Y;
    INC(RISCPTR);
    RISCL[RISCPTR]:=WRITE123UVC;
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDYBUFFER2[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDCBBUFFER2[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.ODDCRBUFFER2[I,1]);
    INC(RISCPTR);
    END;
FOR I:=1 TO 32 DO
    BEGIN
    RISCL[RISCPTR]:=WRITE123Y;
    INC(RISCPTR);
    RISCL[RISCPTR]:=WRITE123UVC;
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENYBUFFER[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENCBBUFFER[I,1]);
    INC(RISCPTR);
    RISCL[RISCPTR]:=INTEGER(@BTBUFFERP.EVENCRBUFFER[I,1]);
    INC(RISCPTR);
    END;
    RISCL[RISCPTR]:=SKIPY4;
    INC(RISCPTR);
    RISCL[RISCPTR]:=SKIPUV;
    INC(RISCPTR);

RISCL[RISCPTR]:=SYNCEVENTSTART;
INC(RISCPTR);
RISCL[RISCPTR]:=0;
INC(RISCPTR);
RISCL[RISCPTR]:=JUMP;
INC(RISCPTR);
RISCL[RISCPTR]:=INTEGER(RISCP)+8;
INC(RISCPTR);

END;


PROCEDURE INITBT848;
BEGIN
//WRITEBT(@BT848P[SRESET],$FF);
//WRITEBT(@BT848L[IFORM],$5B);
WRITEBT(@BT848L[TDEC],$00);
WRITEBT(@BT848L[E_CROP],$23);
WRITEBT(@BT848L[O_CROP],$23);
WRITEBT(@BT848L[E_VDELAY_LO],$20);
WRITEBT(@BT848L[O_VDELAY_LO],$20);
WRITEBT(@BT848L[E_VACTIVE_LO],$40);
WRITEBT(@BT848L[O_VACTIVE_LO],$40);
WRITEBT(@BT848L[E_DELAY_LO],$A0);
WRITEBT(@BT848L[O_DELAY_LO],$A0);
WRITEBT(@BT848L[E_HACTIVE_LO],$00);
WRITEBT(@BT848L[O_HACTIVE_LO],$00);
WRITEBT(@BT848L[E_HSCALE_HI],$03);
WRITEBT(@BT848L[O_HSCALE_HI],$03);
WRITEBT(@BT848L[E_HSCALE_LO],$0A);
WRITEBT(@BT848L[O_HSCALE_LO],$0A);
//WRITEBT(@BT848L[BRIGHT],0);
//WRITEBT(@BT848L[E_CONTROL],$0);
//WRITEBT(@BT848L[O_CONTROL],$0);
//WRITEBT(@BT848L[CONTRAST_LO],$D8);
//WRITEBT(@BT848L[SAT_U_LO],$FE);
//WRITEBT(@BT848L[SAT_V_LO],$B4);
//WRITEBT(@BT848L[HUE],$00);
WRITEBT(@BT848L[E_SCLOOP],$0);
WRITEBT(@BT848L[O_SCLOOP],$0);
//WRITEBT(@BT848L[OFORM],$00);
WRITEBT(@BT848L[E_VSCALE_HI],$0);
WRITEBT(@BT848L[O_VSCALE_HI],$0);
WRITEBT(@BT848L[E_VSCALE_LO],$00);
WRITEBT(@BT848L[O_VSCALE_LO],$00);
//WRITEBT(@BT848L[ADELAY],$7F);
//WRITEBT(@BT848L[BDELAY],$72);
//WRITEBT(@BT848L[ADC],$82);
WRITEBT(@BT848L[E_VTC],$00);
WRITEBT(@BT848L[O_VTC],$00);
//WRITEBT(@BT848L[COLOR_CTL],$10);
//WRITEBT(@BT848L[GPIO_OUT_EN],$00);
//WRITEBT(@BT848L[GPIO_REG_INP],$00);
//WRITEBT(@BT848L[I2C],$00);
END;

PROCEDURE CAPSTART;
BEGIN
CAPSTOP;
INITBT848;
PREPARERISCPRG;
WRITEBT(@BT848L[INT_MASK],$f0000000);//INT MASK
WRITEBT(@BT848L[RISC_STRT_ADD],INTEGER(RISCP));//RISC ADRESS

WRITEBT(@BT848L[COLOR_FMT],YUY2P); //FIFO ENABLE


WRITEBT(@BT848L[GPIO_DMA_CTL],$B1); //FIFO ENABLE        ! TRIGGER POINTS
WRITEBT(@BT848L[GPIO_DMA_CTL],$B3); //RISC/FIFO ENABLE   ! SET TO 32 DWORDS
WRITEBT(@BT848L[CAP_CTL],3);      //CAPTURE CONTROL
INCAPTURE:=TRUE;
END;
PROCEDURE CAPSTOP;
BEGIN
WRITEBT(@BT848L[GPIO_DMA_CTL],0);//RISC/FIFO DISABLE
WRITEBT(@BT848L[CAP_CTL],0);     //STOP CAPTURE
WRITEBT(@BT848L[INT_MASK],$0);//INT MASK
INCAPTURE:=FALSE;
END;

FUNCTION READSTAT:DWORD;
BEGIN
RESULT:=readbt(@BT848L[INT_STAT]) AND $F0000000;
END;
FUNCTION READBT(PT : POINTER):DWORD;
          VAR TEMPDW : DWORD;
BEGIN
ASM
PUSHAD
MOV EDI,PT
MOV EAX,[EDI]
MOV TEMPDW,EAX
POPAD
END;
RESULT:=TEMPDW;
END;

PROCEDURE WRITEBT(PT : POINTER;VALUE : DWORD);
BEGIN
ASM
PUSHAD
MOV EDI,PT
MOV EAX,VALUE
MOV [EDI],EAX
POPAD
END;
END;
end.
